computeReadingMeasures <- function(data) {
	# compute some measures such as first pass fixation (fpf), first fixation duration (ffd), gaze duration (gz) etc.
	#
	# jochen laubrock 2010, 2011
	# this is a candidate function to go into eyetrackR
	
	# TODO: check validity of field names, since we rely on naming convention
	expectedFieldNames <- c("id", "sn", "wn", "wn1.x", "wn2.x", "dur", "ao", "ao1")
	stopifnot(all(expectedFieldNames %in% names(data)))

	# sanity check
	# stopifnot(all(diff(order(data$id, data$sn))==1))
	data <- data[order(data$id, data$sn),]
	invalidIx <- is.na(data$id) | is.na(data$sn)
	if (any(invalidIx)) {data <- data[-which(invalidIx),]}
	
	## frueher: 
	## b$s1 <- b$ao1-(1/b$l1+1)-(b$let+1)
	## b$s2 <- b$ao-(1/b$l2+1)-(1/b$l-b$let)
	## aber bei uns ist leerzeichenmitkodiert. also habe ich die +1 entfernt. (ok?)

	# # Skipping word n-1 (s1) and word n+1 (s2)
	# data$s1 <- data$ao1 - (1/data$l1) - (data$let)
	# data$s1[data$s1 >= 0] <- 1
	# data$s1[data$s1 <  0] <- 0
	# #
	# data$s2 <- data$ao - (1/data$l2) - (1/data$l - data$let)
	# data$s2[data$s2 <  1] <- 0
	# data$s2[data$s2 >= 1] <- 1
	data$s1 <- 0
	data$s1[(data$wn - data$wn1.x) > 1] <- 1
	ix <- which(data$s1==1)
	ix <- ix[ix>1]
	data$s1[ix[data$sn[ix-1]!=data$sn[ix]]] <- 0

	data$s2 <- 0
	data$s2[(data$wn2.x - data$wn) > 1] <- 1
	ix <- which(data$s2==1)
	ix <- ix[ix>1]
	data$s2[ix[data$sn[ix-1]!=data$sn[ix]]] <- 0

	data$s1 <- factor(data$s1)
	data$s2 <- factor(data$s2)

	partition <- as.numeric(as.factor(data$id)) * 1000 + as.numeric(as.factor(data$sn))
	data$fpf <- unlist(tapply(data$wn, partition, firstpass))

	wkey <- as.numeric(as.factor(data$id)) * 100000 + as.numeric(as.factor(data$sn)) *100 + as.numeric(data$wn)
	data$firstFixInSequence <- c(1, diff(wkey)!=0)

	ttot <- unlist(tapply(data$dur, wkey, sum))
	ixIntoData <- match(wkey, as.numeric(dimnames(ttot)[[1]]))
	data$ttot <- ttot[ixIntoData]
	data$ttot[!data$firstFixInSequence] <- NA

	# key <- as.numeric(data$id) * 10000 + as.numeric(data$sn) *100 + data$wn
	gz <- unlist(tapply(data$dur[data$fpf], wkey[data$fpf], sum))
	ixIntoDataGz <- match(wkey, as.numeric(dimnames(gz)[[1]]))
	data$gz <- gz[ixIntoDataGz]
	# check: data[1:20, c("sn", "wn", "dur", "gz")]
	data$gz[!data$firstFixInSequence] <- NA
	data$gz[!data$fpf] <- NA

	data$ffd <- NA
	data$ffd[data$fpf & data$firstFixInSequence] <- data$dur[data$fpf & data$firstFixInSequence] 
	
	nfix <- unlist(tapply(wkey, wkey, length))
	data$nfix <- nfix[ixIntoData]
	data$nfix[!data$firstFixInSequence] <- NA
	# check: data[1:100, c("sn", "wn", "dur", "fpf", "gz", "ffd", "firstFixInSequence", "nfix")]

	nfixgz <- unlist(tapply(wkey[data$fpf], wkey[data$fpf], length))
	data$nfixgz <- nfixgz[ixIntoDataGz]
	data$nfixgz[!data$firstFixInSequence] <- NA
	data$nfixgz[!data$fpf] <- NA
	# check: data[1:100, c("sn", "wn", "dur", "fpf", "gz", "ffd", "firstFixInSequence", "nfix")]

	# da tapply implizit nach wkey ordnet, müssen wir mit order(wkey) indizieren...
	data$fixno <- 1
	data[order(wkey), "fixno"] <- unlist(tapply(data$fixno, wkey, cumsum), use.names=FALSE)

	ixStatsCarrier <- data$fixno==1 & data$fpf==TRUE
	data[!ixStatsCarrier, c("ttot", "gz", "nfix", "nfixgz")] <- NA

	# sel <- parseq(10000*data$id + 100*data$sn + data$wn)
	sel <- parseq(wkey)
	ixLastInFpfSequence <- intersect(which(data$fpf), sel[,2])
	data[ixStatsCarrier, "aogz"] <- data[ixLastInFpfSequence, "ao"]
	data[ixStatsCarrier, "ao1gz"] <- data[ixStatsCarrier, "ao1"]

	data$sfd <- NA
	sfixIx <- ((data$wn>data$wn1.x) | data$wn==1) & ((data$wn < data$wn2.x) | data$wn==data$nw) & data$fpf
	sfixIx[is.na(sfixIx)] <- FALSE
	data$isSingleFix <- sfixIx
	data$sfd[sfixIx] <- data$dur[sfixIx]

	# data$ttot[data$fixno!=1 | data$fpf==FALSE] <- NA
	# data$gz[data$fixno!=1 | data$fpf==FALSE] <- NA
	
	# TODO: ttot, s1, s2, ...
	data
}
