############################
# firstpass
# 	generates index for first-pass fixations for a sentence, given a sequence of fixated words
#   For debugging: 
#		a<-c(1, 2, 2, 5, 1, 1, 3, 1, 3, 5, 5, 6, 6, 6, 2, 3, 4, 5, 6, 7)
#      fpf=c(1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 0, 0, 0, 0, 0, 1)
#
# Jochen Laubrock
firstpass<-function(a)
{
	r <- 0
	cmaxa <- cummax(a)
	fpf <- a>=cmaxa
	# looks straightforward up to here, but: we have to
	# make sure that fixations following regression are not qualified as first pass if they have been visited before
	fpCandidate <- which(diff(fpf)==1) + 1 # first firstpass candidate after regression, 
	if (length(fpCandidate)>0) {
		beenThere <- a[fpCandidate] <= cmaxa[fpCandidate-1] # value was already high at step before, thus we have been there already
		if (any(beenThere)) {
			# find stretch beginning with current invalidly classified fpf, and replace with 0
			ix0 <- fpCandidate[beenThere] 
			for (i in 1:length(ix0)) {
				ix <- ix0[i]:length(cmaxa)
				l <- sum(cmaxa[ix] <= a[ix0[i]])
				ix1 <- ix0[i] + l - 1
				fpf[ix0[i]:ix1] <- FALSE
			}
		}
	}
	
	return(fpf)
}
