projectDir <- "~/Projects/work/eyelinkTest/lesen III"
dataDir <- file.path(projectDir, "neueAuswertung/SacDetec2008/PSC2/fix_seqs_with_R")
ifile <- file.path(dataDir, "PSCII_l3_dvmat.rda") 
load(ifile)

library(MASS)


plot_duration_distributions <- function (data=d.all, whichVar=list(name="GZD", label="gaze duration [ms]"), TMAX = 800, SAVEPLOT = FALSE, OUTPUT_FIG_PATH="." )
{
# TMAX <- 800
# SAVEPLOT <- FALSE
SHOW_NLS_HIST_FITS <- FALSE
# OUTPUT_FIG_PATH <- "/Users/jochen/Projects/presentations/warsaw/Reading/figs"

depvar <- data[!is.na(data[, whichVar$name]) & data[, whichVar$name] <= TMAX, whichVar$name]
# depvar <- d.all[!is.na(d.all[, whichVar$name]) & d.all[, whichVar$name] <= TMAX, whichVar$name]
# depvar <- d.all$GZD[!is.na(d.all$GZD) & d.all$GZD <= TMAX]
xx <- seq(0,TMAX,by=2) # caution: aliasing possible ?

distrs <- c("gamma", "lognormal", "weibull")
densities <- c("dgamma", "dlnorm", "dweibull")
randvals <- c("rgamma", "rlnorm", "rweibull")
fds <- c("fdg", "fdln", "fdw")
ys <- c("yyg", "yyln", "yyw")
cols <- c("red", "green4", "blue")
lws <- c(2, 2, 2)
ltys <- c(1, 1, 1)
parmsstr <- c("","","")

if (SHOW_NLS_HIST_FITS) {
	fds1 <- c("nls.gam", "nls.lnorm", "nls.weib")
	y1s <- c("nls.yyg", "nls.yyln", "nls.yyw")
	a0 = c(4.0, 5.4, 2.0)
	b0 = c(0.01, 0.4, 200.0)
	ltys1 <- c(2, 2, 2)
	parmsstr1 <- c("","","")
}

# the nls version only fits the histogram (1 data point per bin), whereas the fitdistr version fits the raw data
# fitdistr is superior in describing the data, as the ecdf plot shows. interestingly, it fails to produce a good fit to the histogram--why?
# could it be related to the fact that fitting cumulative distributinos is better than fitting densities?

### the actual fitting happens here
yyh <- hist(depvar[depvar>=min(xx) & depvar<=max(xx)], xx, plot=FALSE)
xxx <- yyh$mid
yyy <- yyh$density
for (i in 1:length(distrs)) {
	# raw data fit using fitdistr
	assign(fds[i], fitdistr(depvar, distrs[i]))
	coefs <- coef(eval(parse(text = fds[i])))
	dense <- eval(parse(text = paste( densities[i], "( xx, coefs[[1]], coefs[[2]] )", sep="" )))
	assign(ys[i], dense)
	parmsstr[i] <- sprintf('%s (%4.2f, %4.3f)', distrs[i], coefs[[1]], coefs[[2]])
	# histogram fit using nls
	if (SHOW_NLS_HIST_FITS){
		assign(fds1[i], eval(parse(text = paste("nls(yyy ~ ", densities[i], "(xxx, a, b), start=list(a=", a0[i], ", b=", b0[i], "), model=TRUE)", sep=""))))
		assign(y1s[i], predict(eval(parse(text=fds1[i]))))
		coefs1 <- coef(eval(parse(text = fds1[i])))
		parmsstr1[i] <- sprintf('%s (%4.2f, %4.3f)', distrs[i], coefs1[[1]], coefs1[[2]])
	}
}
### done with fitting 

# if(SAVEPLOT) {pdf(file=file.path(OUTPUT_FIG_PATH, paste(whichVar$name, "_Histogram_0.pdf", sep="")))} else {quartz()}
# plot(yyh, xlab=whichVar$label, freq=FALSE, main="")
# for (i in 1:length(distrs)) {
# 	yy <- eval(parse(text=ys[i]))
# 	parmsstr[i] <- sprintf('%s (%4.2f, %4.3f)', distrs[i], coefs[[1]], coefs[[2]])
# 	if (SHOW_NLS_HIST_FITS){
# 		yy1 <- eval(parse(text=y1s[i]))
# 		lines(xxx, yy1, col = cols[i], lty = ltys1[i], lwd = lws[i])
# 	}
# }
# legend(450, .005, parmsstr, col = cols, text.col = "black", lty = ltys1, lwd = lws, bty = "n")
# if(SAVEPLOT) {dev.off()}

# the above is a short and more extensible version of the more readable  lines below:
# fdg <- fitdistr(depvar, "gamma")
# yyg <- dgamma(xx, coef(fdg)[[1]], coef(fdg)[[2]])
# 
# fdln <- fitdistr(depvar, "lognormal")
# yyln <- dlnorm(xx, coef(fdln)[[1]], coef(fdln)[[2]])
# nls.lnorm <- nls(yyy ~ dlnorm(xxx, a, b), start=list(a=5.4, b=0.4), model=TRUE)
# nls.yyln <- predict(nls.lnorm)# 
#
# fdw <- fitdistr(depvar, "weibull") 
# yyw <- dweibull(xx, coef(fdw)[[1]], coef(fdw)[[2]])
# 
# quartz()
# yyh <- hist(depvar[depvar>=min(xx) & depvar<=max(xx)], xx, freq=FALSE, xlab=whichVar$label, main="")
# lines(xx, yyg, col="red", type="l", lwd=2)
# lines(xx, yyln, col="green", type="l", lwd=2)
# lines(xx, yyw, col="blue", type="l", lwd=2)
# 
# gamparmsstr <- sprintf('(%4.2f, %4.3f)', coef(fdg)[[1]], coef(fdg)[[2]])
# gamtxt <- paste("gamma", gamparmsstr)
# lnparmsstr <- sprintf('(%4.2f, %4.2f)', coef(fdln)[[1]], coef(fdln)[[2]])
# lntxt <- paste("lognormal", lnparmsstr)
# wbparmsstr <- sprintf('(%4.2f, %4.5f)', coef(fdw)[[1]], coef(fdw)[[2]])
# wbtxt <- paste("weibull", wbparmsstr)
# 
# legend(450, .005, c(gamtxt, lntxt, wbtxt), col = c("red", "green", "blue"),
#        text.col = "black", lty = c(1, 1, 1), lwd=c(2, 2, 2))





# if(SAVEPLOT) {pdf(file='/Users/jochen/Projects/presentations/warsaw/Reading/figs/gazeQQ.pdf')} else {quartz()}
# par(mfcol=c(1,3))
# qqplot(yyg, yyh$density, col=cols[1], ylim=c(0,.006), ylab='empirical density', xlab=paste(distrs[1], "fitted density"))
# abline(0,1)
# qqplot(yyln, yyh$density, col=cols[2], ylim=c(0,.006), ylab='empirical density', xlab=paste(distrs[2], "fitted density"))
# abline(0,1)
# qqplot(yyw, yyh$density, col=cols[3], ylim=c(0,.006), ylab='empirical density', xlab=paste(distrs[3], "fitted density"))
# abline(0,1)
# if(SAVEPLOT) {dev.off()}

#TODO: an Struktur anpassen
# if(SAVEPLOT) {pdf(file=file.path(OUTPUT_FIG_PATH, paste(whichVar$name, "_CDF.pdf", sep="")))} else {quartz()}
# plot(sort(depvar), seq(0,1,along.with=depvar), type='l', lwd=2, xlab=whichVar$label, ylab="cumulative density")
# for (i in 1:length(distrs)) {
# 	y <- eval( parse( text = ys[i]) )
# 	lines(xx, cumsum(y*xx)/sum(y*xx), col=cols[i], lty=ltys[i], lwd=lws[i] )
# 	if (SHOW_NLS_HIST_FITS){
# 		y1<- eval( parse( text = y1s[i]) )
# 		lines(xxx, cumsum(y1*xxx)/sum(y1*xxx), col=cols[i], lty=ltys1[i], lwd=lws[i] )
# 	}
# }
# legend(450, .5, c("sample", parmsstr), col = c("black", cols), text.col = "black", lty = ltys, lwd = lws, bty = "n")
# if(SAVEPLOT) {dev.off()}

yps <- c("ypgam", "ypln", "ypweib")
yps1 <- c("ypgam1", "ypln1", "ypweib1")
ypreds <- c("ypredgam", "ypredln", "ypredweib")
ypreds1 <- c("ypredgam1", "ypredln1", "ypredweib1")
coefs <- list()
coefs1 <- list()
for (i in 1:length(distrs)) {
	coefs[[i]] <- coef(eval(parse(text = fds[i])))
	assign(yps[i] , eval(parse(text = paste(densities[i], "(xp , coefs[[i]][[1]], coefs[[i]][[2]])", sep=""))))
	assign(ypreds[i],   eval(parse(text = paste(randvals[i], "(length(depvar), coefs[[i]][[1]], coefs[[i]][[2]])", sep=""))))
	if (SHOW_NLS_HIST_FITS){
		coefs1[[i]] <- coef(eval(parse(text = fds1[i])))
		assign(yps1[i] , eval(parse(text = paste(densities[i], "(xp , coefs1[[i]][[1]], coefs1[[i]][[2]])", sep=""))))
		assign(ypreds1[i],   eval(parse(text = paste(randvals[i], "(length(depvar), coefs1[[i]][[1]], coefs1[[i]][[2]])", sep=""))))
	}
}

if(SAVEPLOT) {pdf(file=file.path(OUTPUT_FIG_PATH, paste(whichVar$name, "_histogram.pdf", sep="")))} else {quartz()}
hist(depvar, 30, freq=FALSE, xlab=whichVar$label, main=paste("Histogram of", whichVar$name))
lines(density(depvar), lwd=1)
for (i in 1:length(distrs)) {
	lines(xp, eval(parse(text=yps[i])), col=cols[i], lwd=lws[i], lty=ltys[i])
	if (SHOW_NLS_HIST_FITS){
		lines(xp, eval(parse(text=yps1[i])), col=cols[i], lwd=lws[i], lty=ltys1[i])
	}
}
legend(TMAX/2 + 50, .004, c("sample", parmsstr), col = c("black", cols), text.col = "black", lty = ltys, lwd = lws, bty = "n")
if(SAVEPLOT) {dev.off()}

if(SAVEPLOT) {pdf(file=file.path(OUTPUT_FIG_PATH, paste(whichVar$name, "_ecdf.pdf", sep="")))} else {quartz()}
plot(ecdf(depvar), xlab=whichVar$label, pch='o', cex=1, main=paste("ecdf(", whichVar$name,")", sep=""))
for (i in 1:length(distrs)) {
	lines(xp, eval(parse(text=paste("cumsum(", yps[i],")", sep=""))), col=cols[i], lwd=3, lty=ltys[i])
	if (SHOW_NLS_HIST_FITS){
		lines(xp, eval(parse(text=paste("cumsum(", yps1[i],")", sep=""))), col=cols[i], lwd=3, lty=ltys1[i])
	}
}
legend(TMAX/2 + 50, .5, c("sample", parmsstr), col = c("black", cols), text.col = "black", pch="o", bty = "n")
if(SAVEPLOT) {dev.off()}

qqps <- c("qqpgam", "qqpln", "qqpweib")
qqps1 <- c("qqpgam1", "qqpln1", "qqpweib1")
if(SAVEPLOT) {pdf(file=file.path(OUTPUT_FIG_PATH, paste(whichVar$name, "_QQ.pdf", sep="")))} else {quartz()}
NSTEPS <- 1000;
ix <- seq(1, length(depvar), by=round(length(depvar)/NSTEPS)) # if all points are plotted, the qq-plot gets too crowded; therefore we select an equidistant 
plot(1,  type="n", xlim=c(0,TMAX), ylim=c(0,TMAX), xlab="predicted quantile", ylab="sample quantile", main=paste('Q-Q plot of', whichVar$name))
for (i in 1:length(distrs)) {
	assign(qqps[i], eval(parse(text=paste( "qqplot(", ypreds[i], ", depvar, xlim=c(0,", TMAX,"), plot.it = FALSE)", sep=""))) )
	xy <- eval(parse(text=qqps[i]))
	# lines(xy$x, xy$y, col=cols[i], lwd=2, lty=ltys[i])	
	points(xy$x[ix], xy$y[ix], col=cols[i], pch='o', cex=0.6)	
	if (SHOW_NLS_HIST_FITS){
		assign(qqps1[i], eval(parse(text=paste( "qqplot(", ypreds1[i], ", depvar, xlim=c(0,", TMAX, "), plot.it = FALSE)", sep=""))) )
		xy1 <- eval(parse(text=qqps1[i]))
		lines(xy1$x, xy1$y, col=cols[i], lwd=2, lty=ltys1[i])
	}	
}
abline(0,1)
legend(0, 600, parmsstr, col = cols, text.col = "black", pch = "o", bty = "n")
if(SAVEPLOT) {dev.off()}
}


# quartz()
# xx <- seq(0,800,2)
# depvarh <- hist(depvar,xx, freq=FALSE)
# plot(depvarh$mid, depvarh$density, type='l', xlab=whichVar$label, ylab="density")
# 
# depvarlong <- sample(depvarh$mid, length(depvar), replace = TRUE, prob = depvarh$density)
# depvarhlong <- hist(depvarlong, xx, plot = FALSE)
# lines(depvarhlong$mid, depvarhlong$density, col="blue")
# # snippet below shows that deviations in the upper part of Q-Q plot are not uncommon with smaller sample sizes
# N <- 500
# # repeatedly execute the next line
# qqplot(sample(depvar,N), sample(depvar,N), xlim=c(0,TMAX), ylim=c(0,TMAX)); abline(0,1)


plot_duration_distributions(data=d.all, whichVar=list(name="GZD", label="gaze duration [ms]"), TMAX = 800, SAVEPLOT = FALSE, OUTPUT_FIG_PATH="/Users/jochen/Projects/presentations/warsaw/Reading/figs")