`parse_msg_file_AAA` <- function(subjectcode, msgdir = './msg/')
{
#-------------------------------------------------------------------
#  FUNCTION parse_msg_file_AAA.m
#  by Jochen Laubrock
#  example parser for SR resarch EyeLink message file
#  (this example from Albrechts Audio Experiment: Laubrock, Cajar, & Inhoff)
#  - parse message file, return relevant info in tab
#		note: becausesentence info (start on screen etc.) was not written to the edf file, 
#	 	we need to load it from an external file and match it to tab by sentence-ID. 

# =======================================================
# = TODO: how do we get corpus info for these sentences =
# =======================================================
#-------------------------------------------------------------------
#  INPUT:
#
#  data from AAA (gaze-contingent sound presentation) experiment, 
#  Eyelink 1000 system, Golm, 2009/10
#
#  subjectcode          subject code
#  *.msg file   message file
#  msgdir            path to file 
#  
#  OUTPUT:
#  matrix 'tab' where each line is a trial = sentence
#  tab(1:ntrials,1)   sentence identification number
#  tab(1:ntrials,2)   x-coordinate of sentence beginning
#  tab(1:ntrials,3)   y-coordinate of sentence beginning (should always be ca. 316 Pixel)
#  tab(1:ntrials,4)   start time of sentence presentation
#  tab(1:ntrials,5)   end time of sentence presentation
#  tab(1:ntrials,6)   x-coordinate of sentence end
#---------------------------------------------------------------------

# # AAA 
# # file header
# 
# MSG	182583 DISPLAY_COORDS 0 0 1023 767
# MSG	182583 RETRACE_INTERVAL  8.32993728099

# # per sentence
#
# MSG	568720 TRIALID 0
# START	568776 	RIGHT	EVENTS
# alternativ
# 	MSG	568813 -7 trialstart fixation trigger
# oder:
# MSG	314789 -7 trialstart fixation trigger
# MSG	318798 0 trialstart fixation not detected, timer
# 
# 	MSG	569186 Start Recording
# 	MSG	569193 -2 sentence displayed
# 		MSG	569193 -2 !V DRAW_LIST ../../runtime/dataviewer/01/graphics/VC_1.vcl
# 		MSG	569194 -1 !V IAREA FILE ../../runtime/dataviewer/01/aoi/IA_1.ias
# 	MSG	569527 0 sound triggered
# 	MSG	569539 sound presentation started
# 	MSG	570994 0 trialend fixation trigger
# 	MSG	570995 End Recording
# END	571013 	EVENTS	RES	  28.28	  27.94
# MSG $TIMESTAMP !V TRIAL_VAR ... (see below)
# MSG $TIMESTAMP TRIAL_RESULT 0
# 
# TRIAL_VAR:
# 	trial_pre 1
# 	sentence_pre Sie: "Wie findest du mein Kleid?"
# 	boundary_pre 5
# 	sound_pre 1-F-Wie.wav
# 	rolle_pre f
# 	calibrationRequested False
# 	playsound 1
# 	sprecher m
# 	group 1
# 	doIncludePretest 1
# 	trial 1
# 	sentence Sie ruft "Hermann..." durch die offene Küchentür.
# 	boundary 9
# 	sound 1-M-Herm.wav
# 	rolle f


# for testing:
# sub{1}='01';
# msgdir <- "/Users/jochen/Projects/work/eyelinkTest/AAA/daten/Experiment/"
# filename <- "01.msg2"

# # parms.R is a text file containing info on PIX_PER_CHAR etc.
# libpath <- '/Users/jochen/Projects/R\ library/eyetrackR/msgparser'
# source(paste(libpath, 'parms.R', sep='/'))
# source('parms.R')

# source('readParms.R')
# parmsfile <- 'parms.dat'
# readParms(parmsfile)


filename = sprintf('%s.msg2',subjectcode);    # file name  
longfilename = sprintf('%s.msg',subjectcode);    # file name  

cat(paste('\nsubject', subjectcode, '\n'))
cat(paste('\t... pre-processing message file', filename, '\n'));   # write formatted data to file

myfile <- readLines(con=file.path(msgdir,filename))
mylongfile <- readLines(con=file.path(msgdir,longfilename))


# =========================
# = remove invalid trials =
# =========================
alltrialfirstlineIx <- grep('TRIALID' , mylongfile, useBytes=TRUE)
# 	potential indices of cancelled trial (due to miscalibration) messages
alltrialcanceledIx <- grep('trialstart fixation not detected', mylongfile, useBytes=TRUE)
#	indices of all TRIAL_RESULT messages
alltriallastlineIx <- grep('TRIAL_RESULT', mylongfile, useBytes=TRUE)

alltrialfirstlineIxShrt <- grep('TRIALID' , myfile, useBytes=TRUE)
# 	potential indices of cancelled trial (due to miscalibration) messages
alltrialcanceledIxShrt <- grep('trialstart fixation not detected', myfile, useBytes=TRUE)
#	indices of all TRIAL_RESULT messages
alltriallastlineIxShrt <- grep('TRIAL_RESULT', myfile, useBytes=TRUE)

ixfalse <- vector()
ixfalseShrt <- vector()
ixvalid <- rep(TRUE, length(mylongfile))
ixvalidShrt <- rep(TRUE, length(myfile))
if (length(alltrialcanceledIx)>0) {
	for (i in seq_along(alltrialcanceledIx)) {
		
		ixfalse[i] <- which(alltrialfirstlineIx < alltrialcanceledIx[i] & alltriallastlineIx > alltrialcanceledIx[i])
		ixfalseShrt[i] <- which(alltrialfirstlineIxShrt < alltrialcanceledIxShrt[i] & alltriallastlineIxShrt > alltrialcanceledIxShrt[i])

		ixvalid[alltrialfirstlineIx[ixfalse[i]]:alltriallastlineIx[ixfalse[i]]] <- FALSE
		ixvalidShrt[alltrialfirstlineIxShrt[ixfalseShrt[i]]:alltriallastlineIxShrt[ixfalseShrt[i]]] <- FALSE
	}
	
	mylongfile <- mylongfile[ixvalid]
	myfile <- myfile[ixvalidShrt]

	alltrialfirstlineIx <- alltrialfirstlineIx[-ixfalse]
	alltriallastlineIx <- alltriallastlineIx[-ixfalse] 
	alltrialfirstlineIxShrt <- alltrialfirstlineIxShrt[-ixfalseShrt]
	alltriallastlineIxShrt <- alltriallastlineIxShrt[-ixfalseShrt] 
	alltrialcanceledIx <- NULL
}


# =============================================
# = parse saccade, fixation, and blink events =
# =============================================
# todo: artefakte durch trialanfang/-ende entfernen
sacmsg <- grep("(ESACC)", mylongfile, useBytes=TRUE, value=TRUE)
sacFields <- strsplit(sacmsg, '[[:blank:]]+')
cRelSacMatrix <- t(sapply(sacFields, "[", 3:11))
sac <- as.data.frame(apply(cRelSacMatrix, 2, as.numeric))
names(sac) <- c("t0", "t1", "dur", "x0", "y0", "x1", "y1", "ampl", "vpeak")

fixmsg <- grep("(EFIX)", mylongfile, useBytes=TRUE, value=TRUE)
fixFields <- strsplit(fixmsg, '[[:blank:]]+')
cRelFixMatrix <- t(sapply(fixFields, "[", 3:8))
fix <- data.frame(apply(cRelFixMatrix, 2, as.numeric))
names(fix) <- c("t0", "t1", "dur", "x", "y", "pa")

blinkmsg <- grep("(EBLINK)", mylongfile, useBytes=TRUE, value=TRUE)
blinkFields <- strsplit(blinkmsg, '[[:blank:]]+')
cBlinkMatrix <- t(sapply(blinkFields, unlist)[3:5,])
blink <- as.data.frame(matrix(apply(cBlinkMatrix, 2, as.numeric), ncol=3))
names(blink) <- c("t0", "t1", "dur")
# todo: handle blinks: mark surrounding saccades (and fixations?) as invalid

# ============================
# = # parse monitor messages =
# ============================
monitor <- list(coords = rep(NA,4), hz=NA)

coordLine <- myfile[1] # MSG	32944 SCREEN_COORDS 0 0 1023 767
coordFields <- strsplit(coordLine, '[[:blank:]]')[[1]]
if (coordFields[3] == "DISPLAY_COORDS") { monitor$coords <- as.double(coordFields[4:7]) } else { stop('first line has wrong format') }

retraceLine <- myfile[2] # MSG	182583 RETRACE_INTERVAL  8.32993728099
retraceFields <- strsplit(retraceLine, '[[:blank:]]')[[1]]
if (retraceFields[3] == "RETRACE_INTERVAL") { monitor$hz <- 1000/as.double(retraceFields[5]) } else { stop('second line has wrong format:\n %s',  myfile[2]) }

# =============================
# = parse trial info messages =
# =============================
# ================================
# = (1) determine relevant lines =
# ================================

# 	indices of all TRIALID messages (recompute indices here, after removal above)
#	(these occur during all trials, regardless whether they were actually presented or not)
alltrialidMsg <- grep('TRIALID', myfile, useBytes=TRUE, value=TRUE)
# 	indices of all trialstart messages
alltrialFixtriggerIx <- grep('trialstart fixation trigger', mylongfile, useBytes=TRUE)
# 	potential indices of cancelled trial (due to miscalibration) messages
alltrialcanceledIx <- grep('trialstart fixation not detected', mylongfile, useBytes=TRUE)
#	indices of all END messages
alltrialendIx <- grep('END', mylongfile, useBytes=TRUE)

alltrialResultMsg <- grep('TRIAL_RESULT', myfile, useBytes=TRUE, value=TRUE)

alltrialFixtriggerMsg <- mylongfile[alltrialFixtriggerIx]
alltrialendMsg <- mylongfile[alltrialendIx]
# 	MSG	569527 0 sound triggered
allsoundtriggeredMsg <- grep('sound triggered', myfile, useBytes=TRUE, value=TRUE)
# 	MSG	569539 sound presentation started
allsoundstartMsg <- grep('sound presentation started', myfile, useBytes=TRUE, value=TRUE)
#	indices of all "sentence displayed" messages
#	(these only occur during actually presented trials)
alltextonMsg <- grep('sentence displayed', myfile, useBytes=TRUE, value=TRUE) 

n1 <- length(alltrialidMsg)
n2 <- length(alltrialendMsg) 
if (n1!=86 || n2!=86)
    stop(sprintf('strange, %d != %d trials !?', n1, n2))
if (n1!=n2){
       stop('strange, different number of trial start and end messages!!!')
} else { 
	cat(paste(n1, 'out of 86 sentences valid\n'))
}



# ===============================
# = (2) extract info from lines =
# ===============================

# define two little helper functions to extract a field from a data set generated from a string by regarding whitespace as field separators
# fieldno is the ordinal number of the field to extract
extractMsgField <- function(msglist, fieldno, toDouble=FALSE)
{
	rows  <- strsplit(msglist, '[[:blank:]]')
	msg <- sapply(rows, '[', fieldno)
	if (toDouble) {
		msg <- as.double(msg)
	}
	msg
}

# TRIAL_VAR messages always have the message (if numeric) in the 6th field
extractTrialvarLoad <- function(msglist, toDouble=FALSE)
{
	msg <- extractMsgField(msglist, 6, toDouble)
	msg
}

extractTimestamp <- function(msglist)
{
	msg <- extractMsgField(msglist, 2, toDouble=TRUE)
	msg
}

# ======================================================
# = below: now use our functions to extract the values =
# ======================================================
# 	MSG	568813 -7 trialstart fixation trigger
trialStartTimestamp <- extractMsgField(alltrialFixtriggerMsg, 2, toDouble=TRUE)
fixtriggerDelay <- extractMsgField(alltrialFixtriggerMsg, 3, toDouble=TRUE)

# END	571013 	EVENTS	RES	  28.28	  27.94
trialEndTimestamp <- extractMsgField(alltrialendMsg, 2, toDouble=TRUE)
# MSG	568720 TRIALID 0
trialIDTimestamp <- extractMsgField(alltrialidMsg, 2, toDouble=TRUE)
trialIDTrial <- extractMsgField(alltrialidMsg, 4, toDouble=TRUE)
# 	MSG	569527 0 sound triggered
trialSoundtriggerTimestamp <- extractMsgField(allsoundtriggeredMsg, 2, toDouble=TRUE)
trialSoundtriggerDelay <- extractMsgField(allsoundtriggeredMsg, 3, toDouble=TRUE)
# 	MSG	569539 sound presentation started
trialSoundplayTimestamp <- extractMsgField(allsoundstartMsg, 2, toDouble=TRUE)
# 	MSG	569193 -2 sentence displayed
trialTextonTimestamp <- extractMsgField(alltextonMsg, 2, toDouble=TRUE)
trialTextonDelay <- extractMsgField(alltextonMsg, 3, toDouble=TRUE)

# =====================================
# = below: extract TRIAL_VAR messages =
# =====================================
# parse TRIAL_VAR messages
allPreTrialMsg <- grep('TRIAL_VAR trial_pre', myfile, useBytes=TRUE, value=TRUE)
allPreSentenceMsg <- grep('TRIAL_VAR sentence_pre', myfile, useBytes=TRUE, value=TRUE)
allPreBoundarylMsg <- grep('TRIAL_VAR boundary_pre', myfile, useBytes=TRUE, value=TRUE)
allPreSoundMsg <- grep('TRIAL_VAR sound_pre', myfile, useBytes=TRUE, value=TRUE)
allPreRoleMsg <- grep('TRIAL_VAR rolle_pre', myfile, useBytes=TRUE, value=TRUE)
allExpPlaysoundMsg <- grep('TRIAL_VAR playsound', myfile, useBytes=TRUE, value=TRUE)
allExpSpeakerMsg <- grep('TRIAL_VAR sprecher', myfile, useBytes=TRUE, value=TRUE)
allExpGroupMsg <- grep('TRIAL_VAR group', myfile, useBytes=TRUE, value=TRUE)
allExpTrialMsg <- grep('TRIAL_VAR trial ', myfile, useBytes=TRUE, value=TRUE)
allExpSentenceMsg <- grep('TRIAL_VAR sentence ', myfile, useBytes=TRUE, value=TRUE)
allExpBoundarylMsg <- grep('TRIAL_VAR boundary ', myfile, useBytes=TRUE, value=TRUE)
allExpSoundMsg <- grep('TRIAL_VAR sound ', myfile, useBytes=TRUE, value=TRUE)
allExpRoleMsg <- grep('TRIAL_VAR rolle ', myfile, useBytes=TRUE, value=TRUE)


# 	trial_pre 1
preTrial <- extractTrialvarLoad(allPreTrialMsg, toDouble=TRUE)
# 	sentence_pre Sie: "Wie findest du mein Kleid?"
preSentence <- gsub("MSG.*TRIAL_VAR sentence_pre ",'',allPreSentenceMsg, useBytes=TRUE)
Encoding(preSentence) <- "latin1"
# 	boundary_pre 5
preBoundaryNo <- extractTrialvarLoad(allPreBoundarylMsg, toDouble=TRUE)
# 	sound_pre 1-F-Wie.wav
preSound <- extractTrialvarLoad(allPreSoundMsg)
# 	rolle_pre f
preRole <- extractTrialvarLoad(allPreRoleMsg)
# 	playsound 1
expPlaysound <- extractTrialvarLoad(allExpPlaysoundMsg, toDouble=TRUE)
# 	sprecher m
expSpeaker <- extractTrialvarLoad(allExpSpeakerMsg)
# 	group 1
expGroup<- extractTrialvarLoad(allExpGroupMsg, toDouble=TRUE)
# 	trial 1
expTrial <- extractTrialvarLoad(allExpTrialMsg, toDouble=TRUE)
# 	sentence Sie ruft "Hermann..." durch die offene Küchentür.
expSentence <- gsub("MSG.*TRIAL_VAR sentence ",'',allExpSentenceMsg, useBytes=TRUE)
Encoding(expSentence) <- "latin1"
# 	boundary 9
expBoundary<- extractTrialvarLoad(allExpBoundarylMsg, toDouble=TRUE)
# 	sound 1-M-Herm.wav
expSound <- extractTrialvarLoad(allExpSoundMsg)
# 	rolle f
expRole <- extractTrialvarLoad(allExpRoleMsg)

# =================================
# = prepare several output tables =
# =================================
x0 <- rep(100, n1) # this is a constant in eike-style experiments
y0 <- rep((monitor$coords[4]+1)/2, n1); # assumption, better to measure!!
x1 <- rep(NA, n1) # this is a constant in eike-style experiments

subjcode <- rep(subjectcode, 86) 

preIx <- 1:18
expIx <- 19:86
trial <- trialIDTrial+1

sentence <- c(preSentence[preIx], expSentence[expIx])
boundary <- c(preBoundaryNo[preIx], expBoundary[expIx])
sound <-  c(preSound[preIx], expSound[expIx]) 
role <-  c(preRole[preIx], expRole[expIx])
speaker <- expSpeaker
speaker[preIx] <- role[preIx]
playSound <- expPlaysound
group <- expGroup

nw <- sapply(strsplit(sentence, '[[:blank:]]'), length)
nlett <- nchar(sentence)

soundplayedIx <- c(rep(TRUE, length(preIx)), rep(FALSE, length(expIx))) | expPlaysound==1
soundtriggerTimestamp <- rep(NA, 86)
soundtriggerTimestamp[soundplayedIx] <- trialSoundtriggerTimestamp
soundtriggerDelay <- rep(NA, 86)
soundtriggerDelay[soundplayedIx] <- trialSoundtriggerDelay
soundplayTimestamp <- rep(NA, 86)
soundplayTimestamp[soundplayedIx] <- trialSoundplayTimestamp

# =================================================================
# = !!!!!!!!!!!!!!!  needs to be checked in EB !!!!!!!!!!!!!!!!!! =
# =================================================================
cat ('!!!!!!!!!!!!!!!  ATTENTION: PIX_PER_CHAR needs to be checked in EB !!!!!!!!!!!!!!!!!!')
PIX_PER_CHAR <- 10 # CHECK!!!
x1 <- x0 + nlett * PIX_PER_CHAR

tab <- data.frame(subjcode, as.matrix(cbind(trial, x0, y0, x1, trialStartTimestamp, trialEndTimestamp)))
names(tab) <- c("subject", "ID", "x0", "y0", "x1", "trialStartTimestamp", "trialEndTimestamp")


trialID <- c(preTrial[preIx],  expTrial[expIx])
trialinfo <- data.frame(subjcode, group, trial, trialID, playSound, role, speaker, boundary, trialStartTimestamp, trialEndTimestamp, soundtriggerTimestamp, soundtriggerDelay, soundplayTimestamp, trialTextonTimestamp, trialTextonDelay, nw, nlett, sentence, sound)

# because of re-calibnrations, trial is not always useful--therefore, we calculate a more sensible trialID variable trialNew by adding 100 to experimental trials (which have trialID<=18 and occur after line 18)
# (this is ad-hoc, but should work in almost all cases)
trialinfo$isExp <- TRUE
trialinfo$isExp[trialinfo$trialID<=18 & ((1:86) <=18)] <- FALSE
trialinfo$trialNew <- 100 * trialinfo$isExp + trialinfo$trialID


S <- list(tab=NULL, trialinfo=NULL,  sac=NULL, fix=NULL,  blink=NULL)
S$tab <- tab
S$trialinfo <- trialinfo
S$sac <- sac
S$fix <- fix
S$blink <- blink
S
}