# Program xSetup1_or  
#
# prepare for distributed-processing analyses
# 

# adapted to AAA, jl 03/10
# enhance readability and general cleanup, jl 02/09
# added predictabilites jl 07/08
# Jochen Laubrock (jl) 08/06, version for oral reading, eyelink 1000
# james ong added a clever speedup (replaced James' trick by use of match, more transparent imho (jl, 03/10))
# most work done earlier by first version provided by R. Kliegl 30/07/05

rm(list=ls())
library('eyetrackR')

projectDir <- "~/Projects/work/eyelinkTest/AAA"
basedataDir <- file.path(projectDir, "daten", "Experiment")
seqDir <- file.path(basedataDir, "fix_seqs")

wbfile <- file.path(basedataDir, 'info', 'AAA_wb.dat')
ifile <- file.path(seqDir, "AAA_allsac_all_RIGHT_inclSpoken.seq")
ofile <- file.path(seqDir, "AAA_fix_RIGHT.rda")

# ======================================================
# =             LOAD AND PREPARE CORPUS                =
# ======================================================
# get word info

# load info about word boundaries in pixels on the subject screen and word frequencies (see create_wb_file.R)
wbtab <- read.table(wbfile, header=TRUE, sep="\t", encoding="UTF-8")

# waste some space: leave some placeholders for to-be-determined corpus info such as initial bigram frequency
corpus <- wbtab[c("sn", "wn", "wbInclSpace")]
corpus$inv_nlett2 <- 1/wbtab$nlett2 #!
corpus$inv_nchr <- 1/wbtab$nchr
corpus$f <- wbtab$frq 
corpus$pred <- NA
corpus$bifr <- NA
corpus$trifr <- NA
corpus$cwfw <- NA

nrow(corpus)
names(corpus) <- c('sn', 'wn', 'wb', 'inv_nlett2', 'inv_nchr', 'f', 'p', 'bifrq', 'trifrq', 'cwfw') 

# ======================================================
# =             GET AND PREPARE FIXATIONS              =
# ======================================================

# GET FIXATIONS (produced by xallsac_AAA.R)
seq <- read.table(file=ifile, header=TRUE)
# > names(seq)
# "id" "sid" "nw" "wn" "let" "fdur" "sacdur" "peakvel" "xampl" "yampl" "x_component" "y_component" "fpos0" "fpos1" "fpos" "firstLastInSentence"

ix <- which(seq[,"fdur"]>=50  & seq[,"fdur"]<=1000)

op <- par(mfcol = c(1,3))
sacdx <- seq[, "xampl"]
sacdxix <- seq[ix, "xampl"]
fixdur <- seq[ix, "fdur"]
hist(sacdxix,breaks=500, xlim=c(-10,20))
hist(sacdxix[abs(sacdxix)<=2], breaks=50, xlim=c(-2,2))
hist(fixdur,500)

# SCREEN DATA, NOTE: ONLY ENTIRE SENTENCES OR IDs CAN BE REMOVED
# Removing fixations within sentences destroys sequence!
# here, nothing is removed

# Remove sentences with max 2 fixated words
sid <- seq[,1,1] * 1000 + seq[,2,1]
six <- parseq(sid)

badsent <- which(six[,3]<=2) 
badfix2 <- vector(mode="numeric", length=sum(six[badsent,2]-six[badsent,1]+1))	
if (length(badsent)>0) {
	nrow(seq)	
	cnt <- 1
	for (i in 1:length(badsent))
	{
		badix <- six[badsent[i],1]:six[badsent[i],2]
		badfix2[cnt:(cnt+length(badix)-1)] <- badix
		cnt <- cnt + length(badix)
	}
	seq <- seq[-badfix2, , ]
	nrow(seq)
}
a <- seq

# quite a few fpos left of sentence:
# summary(seq[is.na(seq$wn),c("fpos", "fpos0", "fpos1", "firstLastInSentence")])
nrow(a)
a <- a[!is.na(a$wn),]
# ======================================================
#     prepare for distributed processing analyses      =
# ======================================================
a <- a[c('id', 'sid', 'nw', 'firstLastInSentence', 'spoken', 'wn', 'let', 'fdur', 'x_component')]
nrow(a)


# INCLUDE INFORMATION ABOUT FIXATED AND NEIGHBORING WORDS
# a[,8:55] <- NA
# names(a) <- c('id',    'sn',  'nw', 
#               'wn',   'let', 'dur', 'ao', 'dir', 'o', 'l',   'f',  
#               'wn1',                                  'l1',  'f1',  
#               'wn2',                                  'l2',  'f2',  
#               'wn3',                                  'l3',  'f3',  
#               'wn4',                                  'l4',  'f4',  
#               'wn1.x','let1','dur1','ao1','dir1','o1','l1.x','f1.x',
#               'wn2.x','let2','dur2','ao2','dir2','o2','l2.x','f2.x',
#               'wn3.x','let3','dur3','ao3','dir3','o3','l3.x','f3.x',
#               'wn4.x','let4','dur4','ao4','dir4','o4','l4.x','f4.x')

a[,10:93] <- NA
names(a) <- c('id',    'sn',  'nw', 'firstLastInSentence', 'spoken', 
              'wn',   'let', 'dur', 'ao', 'dir', 'o', 'l',   'f',   'p',   'bi',   'tri',   'cwfw', 
              'wn1',                                  'l1',  'f1',  'p1',  'bi1',  'tri1',  'cwfw1',
              'wn2',                                  'l2',  'f2',  'p2',  'bi2',  'tri2',  'cwfw2',
              'wn3',                                  'l3',  'f3',  'p3',  'bi3',  'tri3',  'cwfw3',
              'wn4',                                  'l4',  'f4',  'p4',  'bi4',  'tri4',  'cwfw4',
              'wn1.x','let1','dur1','ao1','dir1','o1','l1.x','f1.x','p1.x','bi1.x','tri1.x','cwfw1.x',
              'wn2.x','let2','dur2','ao2','dir2','o2','l2.x','f2.x','p2.x','bi2.x','tri2.x','cwfw2.x',
              'wn3.x','let3','dur3','ao3','dir3','o3','l3.x','f3.x','p3.x','bi3.x','tri3.x','cwfw3.x',
              'wn4.x','let4','dur4','ao4','dir4','o4','l4.x','f4.x','p4.x','bi4.x','tri4.x','cwfw4.x')

# Word numbers
a[a$wn > 1,      "wn1"] <- a[a$wn > 1,      "wn"] - 1
a[a$wn < a$nw ,  "wn2"] <- a[a$wn < a$nw,   "wn"] + 1
a[a$wn > 2,      "wn3"] <- a[a$wn > 2,      "wn"] - 2
a[a$wn < a$nw-1, "wn4"] <- a[a$wn < a$nw-1, "wn"] + 2

cidw <- as.numeric(corpus$sn) * 100 + corpus$wn # cidw = corpus id of word
didw  <- a$sn*100+a$wn       # didw = data id of word

# note: the check below shows that currently there are quite a few words without fixations
# most of these are the last words in sentences--see whether these were excluded in xallsac
# however, a few are also in the middle of a sentence; mostly short function words: skipped?
# naix <- !(cidw %in% didw)
# wbtab[naix,]
# wbtab[naix & wbtab$nw != wbtab$wn,]
# summary(wbtab[naix & wbtab$nw != wbtab$wn,]$frq)
    #  Min.   1st Qu.    Median      Mean   3rd Qu.      Max. 
    # 1.839  1046.000  9370.000 11990.000 23970.000 26530.000 
# summary(wbtab[!naix & wbtab$nw != wbtab$wn,]$frq)
	#      Min.   1st Qu.    Median      Mean   3rd Qu.      Max.      NA's 
	# 8.175e-03 4.319e+01 5.181e+02 3.398e+03 3.377e+03 2.653e+04 1.000e+0

ixIntoCorpus <- match(didw, cidw)
ixIntoCorpusL1 <- match(didw-1, cidw) # word n-1
ixIntoCorpusL2 <- match(didw-2, cidw) # word n-2
ixIntoCorpusS1 <- match(didw+1, cidw) # word n+1
ixIntoCorpusS2 <- match(didw+2, cidw) # word n+2

aCurrentNames <- c("o", "l", "f", "p", "bi","tri", "cwfw")
aL1Names <- c("l1", "f1", "p1", "bi1", "tri1", "cwfw1")
aS1Names <- c("l2", "f2", "p2", "bi2", "tri2", "cwfw2")
aL2Names <- c("l3", "f3", "p3", "bi3", "tri3", "cwfw3")
aS2Names <- c("l4", "f4", "p4", "bi4", "tri4", "cwfw4")
corpusVars <- c("wb", "inv_nlett2", "f", "p", "bifrq", "trifrq", "cwfw")
corpusVars2 <-corpusVars[2:length(corpusVars)]

# TODO: I think we should refrain from using "o" as "wb" intermediately--this has confused me several times. Why not add "wb" to a? Discuss this with the team (jl)

a[, aCurrentNames] <- corpus[ixIntoCorpus, corpusVars]
a[, aL1Names] <- corpus[ixIntoCorpusL1, corpusVars2]
a[, aS1Names] <- corpus[ixIntoCorpusS1, corpusVars2]
a[, aL2Names] <- corpus[ixIntoCorpusL2, corpusVars2]
a[, aS2Names] <- corpus[ixIntoCorpusS2, corpusVars2]

# GENERATE SEQUENCES OF FIVE FIXATIONS, I.E. TWO LAGS, TWO SUCS 
dim1 <- nrow(a)
fillnan <- rep(NA, 12)

# Add lag-1 fixation (44:55)
fromVars  <- c("wn", "let", "dur", "ao", "dir", "o", "l", "f", "p", "bi", "tri", "cwfw" )
lagVars   <- c("wn1.x", "let1", "dur1", "ao1" , "dir1", "o1"  , "l1.x"  , "f1.x"  , "p1.x"  , "bi1.x" , "tri1.x", "cwfw1.x" )
lag2Vars  <- c("wn3.x", "let3", "dur3", "ao3" , "dir3", "o3"  , "l3.x"  , "f3.x"  , "p3.x"  , "bi3.x" , "tri3.x", "cwfw3.x" )
succVars  <- c("wn2.x", "let2", "dur2", "ao2" , "dir2", "o2"  , "l2.x"  , "f2.x"  , "p2.x"  , "bi2.x" , "tri2.x", "cwfw2.x" )
succ2Vars <- c("wn4.x", "let4", "dur4", "ao4" , "dir4", "o4"  , "l4.x"  , "f4.x"  , "p4.x"  , "bi4.x" , "tri4.x", "cwfw4.x" )

lag <- a[,fromVars]              # copy "a" into "lag"
lag <- lag[-dim1,]               # delete last row of "lag"
lag <- rbind(fillnan, lag)       # insert NaN at beginning = shift down!
a[,lagVars] <- lag               # put lag=down shifted "a" next to "a"
rm(lag)

# Add suc-1 fixation (56:67)
suc <- a[,fromVars]
suc <- suc[-1,]
suc <- rbind(suc, fillnan)	
a[,succVars] <- suc			
rm(suc)

# Add lag-2 fixation (68:79)
lag2 <- a[,lagVars]
lag2 <- lag2[-dim1,]
lag2 <- rbind(fillnan,lag2) 
a[,lag2Vars] <- lag2
rm(lag2)

# Add suc-2 fixation (80:91)
suc2 <- a[,succVars]
suc2 <- suc2[-1,]
suc2 <- rbind(suc2, fillnan)
a[,succ2Vars] <- suc2
rm(suc2)

# Index for sentence boundaries
# sid <- a[,1]*1000+a[,2]  
sid <- a$id*1000 + a$sn
six <- parseq(sid)               # Start and n of fix for each sentence

# Replace impossible lag and succ values with NaNs
a[six[,1],c(lagVars, lag2Vars)] <- NA 		#  First fix of sentence: no lag-1
a[six[,1]+1,c(lag2Vars)] <- NA 				#  Second fix of sentence: no lag-2
a[six[,2],c(succVars, succ2Vars)] <- NA 	#  Last fix of sentence: no suc-1
a[six[,2]-1,c(succ2Vars)] <- NA 			#  Last-1 fix of sentence: no suc-2


# # Absolute letter position of fixations on line, replaced later with "o" 
# wbnames <- c("o","o1","o2","o3","o4")
# wbpos <- match(wbnames, names(a) )
# #wbpos <- c(9, 29, 37, 45, 53)
# lettpos <- wbpos-4
# lengthpos <- wbpos+1
# j<-0
# for (i in wbpos )   
# {   
# 	j <- j+1
#     a[,i] <- a[,i]+a[,i-4]         # "9" contains wb, "5" cont letter position
# }  


# (jl) more concise replacement for the above,  ok?
# Absolute letter position of fixations on line, replaced later with "o" 
wbnames <- c("o","o1","o2","o3","o4")
lettnames <- c("let","let1","let2","let3","let4")
lengthnames <- c("l","l1","l2","l3","l4")
for (j in 1:length(wbnames))   
{   
    a[,wbnames[j]] <- a[,wbnames[j]] + a[,lettnames[j]]         
}

pix <- c("o2", "o", "o4", "o1")
aos <- c("ao", "ao1", "ao2", "ao3")
dirs <- c("dir", "dir1", "dir2", "dir3")
for (j in 1:4)
{   
    a[,aos[j]]   <- a[,pix[j]] - a[,wbnames[j]] # Signed amplitude
    a[,dirs[j]] <- sign(a[,aos[j]])          # Strip off sign and store dir
    a[,aos[j]]   <- abs(a[,aos[j]])           # Absolute amplitude in ao
}

a$dir4 <- sign(a$ao4)  # For n+2 fix use dX amplitude, not letter distance
a$ao4  <- abs(a$ao4)   # For n+2 fix use dX amplitude (here: degrees visual angle), not letter distance

# summary(a[a$dir>0,]$ao,100)

# # ????
# a$ao4 <- c(a$ao2[-1],NA)  # delete first row to get ao2 of subsequent fixation
# a$dir4 <- c(a$dir2[-1],NA)
# # ????


# Convert letter position to relative position within word:
# 0 = space before word, .50 = middle letter, 1 = last character (could be punct.)
# a[,wbpos] <- a[,lettpos]/(1/a[,lengthpos]+1)
a[,wbnames] <- a[,lettnames] / ( 1 / a[,c("l", "l1.x", "l2.x", "l3.x", "l4.x")] + 1)

# Formula looks funny because "l" is stored as reciprocal!

save(a,file=ofile)
