# Program xSetup2_l3.R
# second stage of lesen III with psc2 analyses
# centering and transformation of predictors
# limit further analyses to valid first pass fixations
# additional constraint: fixation durations of each triplet fixation between 50 and 750 ms
#
# added predictabilities jochen laubrock 07/08
# jochen laubrock 12/06
# jochen laubrock 08/06
# after r. kliegl 29/04/2006

# 
rm(list=ls())
library('eyetrackR')

projectDir <- "~/Projects/work/eyelinkTest/lesen III"
dataDir <- file.path(projectDir, "neueAuswertung/SacDetec2008/PSC2/fix_seqs_with_R")

ifile1 <- file.path(dataDir, "PSCII_l3_fix_RIGHT_binoc.rda") 	# file containing right-eye fixations in distributed-processing format
ifile2 <- file.path(dataDir, "PSCII_l3_fix_index_logical.rda") 	# file containing logical indices
ofile1 <- file.path(dataDir, "PSCII_l3_abcd_RIGHT_binoc.rda")         # save working files

load (ifile1)  # Use only right-eye data for DistribProc ...
nrow(a)
load (ifile2)  # Extract sf according to KNE2006

# Use index to filter relevant fixations
# NOTE: here we use all valid first pass fixations. In KNE2006, a different index (triplets) was used !
a <- a[vFirstp$RIGHT,]
nrow(a)

# Save reduced "a" and new variables in "b"
# Local additional filtering and selection of variables for analysis
relvarnames <- c("id", "sn", "nw", "wn", "let", "dur", "ao", "o", "l", "f", "p",  "l1", "f1", "p1", "l2", "f2", "p2", "let1", "dur1", "ao1", "o1", "let2", "dur2", "o2") 
b <- a[,relvarnames] 

b$id <- factor(b$id)
b$sn <- factor(b$sn)
b$nw  <- as.numeric(b$nw)
b$wn  <- as.numeric(b$wn)
b$let <- as.numeric(b$let)
b$dur <- as.numeric(b$dur)

ixValidDurationOfNeigboringFixations <- a$dur1>=51 & a$dur1<=750 & a$dur2>=51 & a$dur2<=750
b <- b[ixValidDurationOfNeigboringFixations, ,drop=TRUE] 
# Alternative for geting rid of unused factor levels (Ripley)
#b[] <- lapply(b,function(x) x[drop=TRUE])

# Skipping word n-1 (s1) and word n+1 (s2)
b$s1 <- b$ao1 - (1/b$l1 + 1) - (b$let + 1) 
b$s1[b$s1 >= 0] <- 1
b$s1[b$s1 <  0] <- 0
#
b$s2 <- b$ao - (1/b$l2 + 1) - (1/b$l - b$let)
b$s2[b$s2 <  1] <- 0
b$s2[b$s2 >= 1] <- 1

# b$s1 <- factor(b$s1) # why?
# b$s2 <- factor(b$s2) # why?

# Center continuous predictors relative GRAND MEN of fixations, except o, save in matrix c

# cen <- c(3:5, 7, 9:13, 15:19, 21:25, 27, 29, 31)
# cen <- c("nw","wn","let","ao","l","f","l1", "f1","l2","f2","let1","ao1","let2")
cen <- c("nw", "wn", "let", "ao", "l", "f", "l1", "f1", "l2", "f2", "let1", "ao1", "let2")
c <- b

c[ ,cen]  <- scale(c[ ,cen], scale=F)

c$dur  <- log(c$dur)
c$dur1 <- log(c$dur1)
c$dur2 <- log(c$dur2)

c$o   <- c$o -  0.5  # Preserve center-of-word metric: O = middle letter
c$o1  <- c$o1 - 0.5
c$o2  <- c$o2 - 0.5

c$oq  <- c$o * c$o
c$lf  <- c$l * c$f
c$f1f <- c$f1 * c$f
c$ff2 <- c$f * c$f2
c$lf2 <- c$l * c$f2

c$s1 <- as.numeric(as.character(c$s1))
c$s2 <- as.numeric(as.character(c$s2))


# Center continuous predictors relative to mean of IDs, except "o", "o1", "o2"
# This is not as critical as centering at the FIX-level; may be needed for interactions
d <- b
idx <- parseq(as.numeric(as.character(d$id)))
ids <- unique(d$id)

for (i in 1:length(ids) ) {
	d[idx[i,1]:idx[i,2], cen] <- scale(d[idx[i,1]:idx[i,2], cen], center=TRUE, scale=FALSE)
}

d$dur  <- log(d$dur)
d$dur1 <- log(d$dur1)
d$dur2 <- log(d$dur2)

d$o   <- d$o -  0.5  # Preserve center-of-word metric: O = middle letter
d$o1  <- d$o1 - 0.5
d$o2  <- d$o2 - 0.5

d$oq  <- d$o*d$o
d$lf  <- d$l*d$f
d$f1f <- d$f1*d$f
d$ff2 <- d$f*d$f2
d$lf2 <- d$l*d$f2

d$s1 <- as.numeric(as.character(d$s1))
d$s2 <- as.numeric(as.character(d$s2))

# Output
save(a,b,c,d, file=ofile1)
