rm(list=ls())
local_lib_path="~/Projects/work/PSC_official/"

projectDir <- "~/Projects/work/eyelinkTest/AAA"
basedataDir <- file.path(projectDir, "daten", "Experiment")
seqDir <- file.path(basedataDir, "fix_seqs")

ifile <- file.path(seqDir, "AAA_abcd_RIGHT.rda")
ofile <- file.path(seqDir, "allAAA.rda")         # save working files


load(ifile)        # merged "b" and "c" files

# file for id-level analyses; note use "d" values ...
# ... for numeric cnd, x, x1, x2, s1, s2, oq, lf, f1f, etc.
attach(b)
b.id <- aggregate(cbind(dur.id=dur, ldur.id=log(dur),
     f.id=f,   l.id=l, p.id=p,
     f1.id=f1, l1.id=l1, p1.id=p1,
     f2.id=f2, l2.id=l2, p2.id=p2,
    ao1.id=ao1, o.id=o, oq.id=d$oq, ao.id=ao,
    lf.id=d$lf, f1f.id=d$f1f,
    s1.id=d$s1, s2.id=d$s2),
    by = list(id=id), mean, na.rm=TRUE)
detach()


# Include estimate of first-pass skipping rate
b.id$s.id <- (b.id$s1+b.id$s2)/2

# Center the continous variables
d.id <- b.id
contcols<-c("dur.id", "ldur.id", "f.id","l.id", "p.id",
"f1.id", "l1.id", "p1.id", "f2.id","l2.id", "p2.id", "ao1.id","o.id","oq.id","ao.id",
"lf.id","f1f.id", "s1.id","s2.id","s.id")
d.id[ ,contcols] <- scale(d.id[ ,contcols], scale=FALSE)

# Generate PCs and Nfix 
attach(d.id)
pca.mat=cbind(l1.id, f1.id, s.id, l.id, f.id)
detach()

library(MASS)
summary(id.pca <- princomp(pca.mat, cor=TRUE))
plot(id.pca)
loadings(id.pca)
id.pc <- predict(id.pca)
eqscplot(id.pc[,1:2], xlab="1st PC", ylab="2nd PC")

d.id$pc1 <- id.pc[,1]
d.id$pc2 <- id.pc[,2]

# Use the id-level values in b.id +pc's as covariates in fix-level b, c, and d
# c: fix-level predictors centered at grand mean, but use id-level stuff from d
# d: fix-level predictors centered at ID mean

#TODO: here we need different numbers
newcols_c <- (ncol(c)+1):(ncol(c)+length(contcols))
c[,newcols_c] <- NA
names(c)[newcols_c] <- names(d.id[,contcols])
newcols_d <- (ncol(d)+1):(ncol(d)+length(contcols))
d[,newcols_d] <- NA
names(d)[newcols_d] <- names(d.id[,contcols])

for (i in 1:nrow(d.id))
{	
	matches  <- which(d$id == d.id$id[i])   # match id in id-level df and fix-level df
 	c[matches,newcols_c] <- d[matches,newcols_d] <- d.id[i,contcols]  # copy the id-level info to fix-level df
}

save(b, c, d, b.id, d.id, file=ofile)

