# =======================================================
#  xallsac_AAA.R                               
#  by jochen laubrock                                 
#  cleaned-up and simplified version of antjes stuff  
#  for AAA (Albrecht Audio Experiment)      
# =======================================================
#
# a real-life example of how to get from raw data to a fixation sequence
#
# The file is NOT meant to be executed with demo(xallsac), but rather 
# to be studied and used as a starting point for own analyses programs
#
# still needs further cleanup
# TODO 
# - write all tables only at the end
# - think about how to make this more generic (provide an interface)
# - provide option to use SR parser if available
#-------------------------------------------------------------------
#
#  INPUT:           data from reading experiment:
#                   *.msg, *.msg2	for a given subject, text file with messages regarding the experiment 
#									(generated by R script preprocess())
#                   *.dat           for a given subject, file with raw data (also generated by preprocess())
#
#  OUTPUT:          .seq file containing sequence of fixations
#
#  FORMAT:          every row is a fixation
#                   16 columns
#                   1) subject id,
#                   2) sentence id,
#                   3) number of words
#                   4) word number (fixated word)
#                   5) letter number (fixated letter), currently NA
#                   6) fixation duration [ms]
#                   7) duration of the following saccade [ms]
#                   8) peak velocity [deg/ sec]
#                   9) dX x amplitude of the following saccade [deg]
#                   10) dY y amplitude of the following saccade [deg]
#                   11) dx x horizontal component
#                   12) dy y vertical component
#                   13) x position of gaze, in screen coordinates (pixels) at beginning of fixation, fpos_a(k)
#                   14) x position of gaze, in screen coordinates (pixels) at end of fixation, fpos_b(k)
#                   15) x position of gaze, in screen coordinates (pixels), averaged across fixation, fpos(k)
#                   16) first (1) vs. last (2) fixation within a sentence (0 otherwise)
#
#  dependencies:  readSubjects.R, parse_msg_file_AAA.R, getMissingData.R, vecvel.R, microsacc.R, binsacc.R, wordLetterPos.R, treatBadData.R
#---------------------------------------------------------------------
# screen resolution during (this) reading experiment: 1024 x 768 Pixel
# distance between screen monitor and subject: 60 cm
#
# NB: my recommendation: in reading, always use binsacc 

# rm(list=ls())
library(eyetrackR)
# provides 'readSubjects.R', 'parms1.R', 'treatBadData.R', 'wordLetterPos.R'

source('/Users/jochen/Projects/R library/potsdam/eyetrackR/eyetrackR/demo/parse_msg_file_AAA.R')
# 'parse_msg_file_AAA.R',

# =============================================================================
# =                                                                           =
# =                              PARAMETER BLOCK                              =
# =                (adapt the following lines to your needs)                  =
# =                                                                           =
# =============================================================================

exp_str <- 'AAA'   		# string characterizing the experiment

# ===========================
# = define some directories =
# ===========================
basepath <- '~/Projects/work/eyelinkTest/AAA'
projectpath <- basepath
# paths for raw input data
basedatapath <- file.path(basepath, 'daten', 'Experiment')
msgfilepath <- file.path(basedatapath, 'msg')
datfilepath <- file.path(basedatapath, 'dat')
# where the sequence files will be stored
outputpath <- file.path(basedatapath, 'fix_seqs') 
# where info on missing and/or 'bad' data will be saved
missingpath <- file.path(outputpath, 'missings') 

oldDir <- setwd(basedatapath)
# =============================================================================
# =                                                                           =
# =                            END PARAMETER BLOCK                            =
# =                                                                           =
# =============================================================================
if (!file_test("-d", basepath)) {stop('please correct basepath')}
if (!file_test("-d", msgfilepath)) {stop('please correct msgfilepath')}
if (!file_test("-d", datfilepath)) {stop('please correct datfilepath')}
if (!file_test("-d", outputpath)){dir.create(outputpath)}
if (!file_test("-d", missingpath)){dir.create(missingpath)}


print('========================================================================')
print('This file is NOT meant to be executed with demo(xallsac_AAA), but rather')
print('to be studied and used as a starting point for own analyses programs')
print('========================================================================')

print('\n\n+++ conversion of raw data to fixation sequences +++\n');


# Read subject list (and check for availability of associated raw data sets)
subjects <- readSubjects(exp_str, dir=basedatapath, msgdir=msgfilepath, datdir=datfilepath)
mis_collect <- NULL

# =======================
# = prepare output files 
# =======================

nos <- length(subjects) # number of subjects
# construct file names and open output file for fixation sequence data

sacdefLbl <- 'RIGHT'
outfilename <- file.path(outputpath, sprintf('%s_allsac_all_%s.seq', exp_str, sacdefLbl))
outfile <-  file(outfilename, open = 'w');

# collection matrices for 'missing data' (same for both eyes)
mis_col_s_trial_id <- NULL
mis_col_NrPerSubject <- NULL

# collection matrices for 'bad data' (same for both eyes)
mis2_col_s_trial_id <- NULL
mis2_col_NrPerSubject <- NULL

data(parms1000)
cat('\nPARAMETERS FOR (MICRO)SACCADE DETECTION:\n');
cat(sprintf('velocity type %i, velocity threshold i, minimum duration of saccade %i samples\n', VTYPE, VTHR, MINDUR))
cat(sprintf('sampling rate %i, PPC %i', SAMPLING, PIX_PER_CHAR))


# headerStr <- sprintf('%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\n',  
# 	"subjID", "id", "nwords", "w", "l", "fixdur", "sacdur", 
# 	"vpeak", "dX", "dY", "dx", "dy", 
# 	"fpos_a", "fpos_b", "fpos", "FirstLast")
headerStr <- sprintf('%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\n',  
"id","sid","nw","wn","let","fdur","sacdur","peakvel", "xampl", "yampl", "x_component", "y_component", "fpos0", "fpos1", "fpos", "firstLastInSentence", "isSpoken")

cat(headerStr, file=outfile, append = TRUE)
	
# ======================
# = loop over subjects =
# ======================

for (s in 1:nos) {  # loop over subjects =
	sub <- subjects[s] # Create character array (string)
	
	subjID <- as.numeric(sub) # needs to be adapted according to how output file names are constructed
	
	# ====================================================
	# 1)    get relevant information from message file
	# 		(relies on function parse_msg_file_AAA, which
	# 		needs to be adapted to each new experiment)
	# ====================================================
	S <- parse_msg_file_AAA(sub,msgfilepath)   # call function parse_msg_file to parse message file 
	tab <- S$tab
	info <- S$trialinfo
	# add 100 to experimental trials, and use trialID rather than trial to accommodate re-calibrations 
	info$isExp <- TRUE
	info$isExp[info$trialID<=18 & ((1:86) <=18)] <- FALSE
	info$trialNew <- 100 * info$isExp + info$trialID
	
	# vector of sentence lengths in characters
	nchrSentences <- nchar(as.character(info$sentence))
	# list (length n_sentence) of vectors of word lengths (including punctuation) in characters
	nlett <- sapply(strsplit(as.character(info$sentence), '[[:blank:]]'), nchar)
	
	cat(sprintf('\n\t... loading raw data for %s (No. %i, subject %i) \n', sub, s, subjID))  
	datfile <- file.path(datfilepath, sprintf('%s.dat',sub))
	datfilesize <- file.info(datfile)$size
	# monocular recording of only four data columns	
	# TODO: extract field names and number of columns from edf/msg file
	data <- read.table(datfile, nrows=datfilesize, col.names=c('t', 'xr', 'yr', 'pr'))   # load file with raw data
	# !!!! note: the nrows default in the above is dangerous: if file is longer,  it is truncated,  which leads to "no samples found " messages for trials late in the experiment
	# however,  without providing nrows,  read.table is very slow
	# make sure to use a generous estimate
	# new: guesstimate nrow from file size
	
	cat(sprintf('\t... raw data for %s (subject %i) are converted to fixation sequences\n', sub, s))
	ttime <- data$t # time stamp from 1. column of raw data matrix
	for (trial in 1:nrow(tab))  {# loop over trials
		# id <- tab[trial, 'ID']      # get sentence ID 
		id <- info[trial, 'trialNew']      # get sentence ID 
		nwords <- info$nw[trial]   # number of words with sentence

		sentenceWL <- nlett[[trial]]
		
		# find rows representing the current trial and/ or sentence (logical index)
		indx <- ttime >= tab[trial, "trialStartTimestamp"] & ttime <= tab[trial, "trialEndTimestamp"]
		# treat missing-data trials
		if (sum(indx)<=1) {
			cat('no / too few samples found for subject', sub, 'trial',trial,'\n'); # print error message on screen if no data points found
			mis2_col_s_trial_id <- rbind(mis2_col_s_trial_id, c(s, trial, id))    # collect them
			next # skip over remainder of trial loop
		}
		time <- ttime[indx]  # consider only relevant time stamps
		
		# new: add and process info about word boundaries 
		LEFT_SENTENCE_OFFSET <- tab[trial,]$x0
		RIGHT_SENTENCE_OFFSET <- tab[trial,]$x1
		dx <- RIGHT_SENTENCE_OFFSET - LEFT_SENTENCE_OFFSET
		nchr <- nchrSentences[trial]
		PIX_PER_CHAR <- dx/nchr
		rightSentenceBorder <- LEFT_SENTENCE_OFFSET + nchr * PIX_PER_CHAR  
		
# ==========================================
# = TODO: compare with info from IAS files =
# ==========================================


		# ====================================================
		# 2)    Detection of missing values
		# ====================================================
		if (RIGHT_SENTENCE_OFFSET != rightSentenceBorder) {
			cat(sprintf('different corpus? check vp %s trial %d sentence %d\n', sub, trial, id))
		}

		data_trial <- data[indx,]
		MSbeforeSentence <- info[trial,"trialTextonTimestamp"] - info[trial,"trialStartTimestamp"]
		data_trial[1:MSbeforeSentence, 2:ncol(data_trial)] <- NA
		# define missing data, ignore missings after right sentence border
		res <- getMissingDataAAA(data_trial, LEFT_SENTENCE_OFFSET, RIGHT_SENTENCE_OFFSET, PIX_PER_CHAR)
		missing <- res$missing
		data_trial <- res$data_trial
		
		validIx <- !is.na(data_trial[,2])
		data_trial <- data_trial[validIx, ]
		time <- time[validIx]
		
		if (length(missing)>0)  {  # if there are missings, then collect them
			mis_col_s_trial_id = rbind(mis_col_s_trial_id, c(s, trial, id))  # subject id, trial no., sentence no.
		} else { # if there are no missings with this trial, then continue ...
			# ====================================================
			# 3)    Detection of all saccades
			# ====================================================
			# This example shows monocular saccade detection.
			# Tor binocular analysis, make sure that this is repeated for left eye, and that binsacc is run
			# (see xallsac<- l3.R)
			xyr <- DEG_PER_PIX * data_trial[, c("xr", "yr")] # x- and y-position for RIGHT eye
			vr <- vecvel(xyr, SAMPLING, VTYPE);        # computation of velocity components (x & y) for RIGHT eye data
			sacr <- microsacc(xyr, vr, VTHR, MINDUR, removeGlissades=TRUE, DTGLISS=50, glissadesBelongToFixation=TRUE, discardMS=TRUE, MSamplCrit=0.30);  # RIGHT eye,output: '7 x number of saccades'-matrix
			# note: here we are using default settings for parameters except MSamplCrit, but we make our choice explicit here for purposes of documentation  
			
			# (1) onset of saccade, (2) end of saccade, (3) peak velocity,
			# (4) horizontal component (dx = distance between saccade onset and offset) ,
			# (5) vertical component (dy)
			# (6) horizontal amplitude dX,
			# (7) vertical amplitude dY
			# amplitude is maximum expansion between saccade onset and offset
			
			readsac <- sacr$sac
			if (nrow(readsac)==0) { # if no saccades detected
				#visualize_empty_sac(s,trial,xr,vr);   # visualize trial
				mis_collect <- rbind(mis_collect, c(s, trial, id))    # collect them
			} else {
# =============================================
# 		TODO: insert glissade correction here =
# =============================================				
				
				# initialize 
				fixdur <- NULL          # for the current trial, initialize array for collection of fixation durations
				sacdur <- NULL          # for the current trial, initialize array for collection of fixation durations
				w  <- NULL              # ditto for word assignment
				l  <- NULL              # ditto for letter assignment
				lp  <- NULL             # ditto for character assignment (Chinese readers)
				chz  <- NULL            # ditto for character zone assignment (Chinese readers)
				fpos <- NULL            # ditto for MEAN fixation position
				fpos_a  <- NULL         # ditto for fixation position at the START of fixation
				fpos_b  <- NULL         # ditto for fixation position at the END of fixation
				a  <- NULL              # ditto for ONSET of fixation (relative to sentence onset)
				b  <- NULL              # ditto for OFFSET of fixation (relative to sentence onset)
				bad  <- NULL            # ditto for indices of 'bad' fixation positions
				nsac_start <- 1         # number of (valid) start sac./ fix. is set to 1
				nsac_end <- 1  # 
				nsac <- nrow(readsac)   # number of saccades
				
				# get x-position of the eye [pixels], for the whole trial
				xpix <- data_trial$xr
				
				if (readsac[1,1] == 1) {
					# maybe I need to check sac(1,1) only --> start/stop of L and R appears to be always the same
					firstsac <- readsac[1,2]  # keep saccade end time
					readsac <- readsac[-1,]      # remove saccade ( Matlab: readsac(1,) <- [] )
					cat(sprintf('\tFirst saccade starts at time stamp 1 (trial: %i)!\n', trial))
				} else {
					firstsac <- 0
				}
				nsac_end <- nrow(readsac)   # nsac = number of (reading) saccades
				
				if (nrow(readsac) > 0) {
					for (k in 1:nsac_end)  {          # use ALL saccades in order to determine fixations => for every saccade ...
						# ====================================================
						# 4)    Determine fixations
						# ====================================================
						# note: fixations are stored next to the outgoing saccade
						if (k==1)  {
							# a(k) = 1;                 # (ALTE VERSION) special case of 1. fixation: there is no preceding saccade -> start = 1 
							a[k] <- firstsac + 1         # (Christiane) special case of 1. fixation: there is no preceding saccade -> start = 1
						} else {
							a[k] <- readsac[k-1,2] + 1   # a = start of fixation = end of preceding saccade (k-1)
						}
						b[k] <- readsac[k,1] - 1     # b = end of k. fixation = start of k. saccade
						if (b[k] < 30) # can happen if first saccade in trial is within first few samples
							fixdur[k] <- NA
						else
							fixdur[k] <- 1000/SAMPLING * (b[k] - a[k] + 1)    # computation of fixation duration
						stopifnot (fixdur[k]>=30 | is.na(fixdur[k]))
						# if (fixdur[k]<30) {browser()}
						sacdur[k] <- ( readsac[k,2] - readsac[k,1] + 1 ) * 1000/SAMPLING   # duration of following saccade
						# k now indexes FIXATIONS (instead of saccades)
					
						# ====================================================
						# 5)    Analysis of fixation location
						# ====================================================
						#TODO (jochen): here we sometimes have isempty(xpix(a(k):b(k))). Check why, and catch !! 
						# my guess: it's related to premature deletion of data in getMissingData
						thisFixX <- xpix[a[k]:b[k]]
					
						if ( length(thisFixX) == 0 ) {
							dummy <- NULL #dummy used for setting breakpoint
							print(paste('empty again for subject:', sub, 'trial:', trial))
						}
					
						fpos[k] <- mean(thisFixX); # MEAN fixation position
						# a = ONSET of fixation (time stamp relative to sentence onset)
						# b = OFFSET of fixation (time stamp relative to sentence onset)
					
						fpos_a[k] <- xpix[a[k]]  # position of the eyes at the START of fixation
						fpos_b[k] <- xpix[b[k]]  # position of the eyes at the END of fixation
					
						# word and letter assignment
						res <- wordLetterPos(fpos[k]-LEFT_SENTENCE_OFFSET, sentenceWL, PIX_PER_CHAR)
						w[k] <- res$w
						l[k] <- res$l
					
					} # end of loop for k=1:nsac_end (fixation analysis)
				
					# ====================================================
					# 6)    Treatment of data left and/or right of sentence  
					# ====================================================
					# NB! the fixation location analysis always provides an assignment!
					# However, if the gaze was actually LEFT of the sentence, the fixation is assigned to the FIRST word (and first letter), and
					# if the gaze was actually RIGHT of the sentence, the fixation is assigned to the LAST word (last letter) of the sentence,
					# which apparently doesn't make sense. (we now treat this in wordLetterPos and assign NA (jl))
					# Therefore, it is implemented that 'fpos' must not be negative and must not be greater than the right sentence border
					# tolerance region: LCRIT letters
					LCRIT <- 3
					excludeSent <- 0
					ixBad <- fpos < LEFT_SENTENCE_OFFSET - LCRIT*PIX_PER_CHAR | fpos > rightSentenceBorder - LEFT_SENTENCE_OFFSET + LCRIT*PIX_PER_CHAR 
					if (any(ixBad)) {
						bad <- which(ixBad)
						res <- treatBadData(bad, fpos, rightSentenceBorder - LEFT_SENTENCE_OFFSET, PIX_PER_CHAR)
						nsac_start <- res$nsac_start
						nsac_end <- res$nsac_end
						excludeSent <- res$excludeSent
					}
				} else {excludeSent = 1}
				if (excludeSent == 1)  {  # if sentence is to be excluded
					mis2_col_s_trial_id <- rbind(mis2_col_s_trial_id, c(s, trial, id))    # collect them
					# subject id, trial nr., sentence nr.
				}
				
				if (excludeSent == 0) { # if sentence is not to be excluded
					# ====================================================
					# 7)    Write out fixation sequence
					# ====================================================
					
					for (k in nsac_start:nsac_end) { # for the first to last fixation ...
						if (k == 1) {
                               FirstLast <- 1      # marker for the FIRST fixation in a sentence
                           } else if (k == nsac_end) {
                               FirstLast <- 2  # marker for the LAST fixation in a sentence
                           } else {
                               FirstLast <- 0  # neither the first nor last fixation...
                           } 
						# resultStr <- sprintf('%i\t%i\t%i\t%i\t%f\t%i\t%i\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%i\n',  
						# 	s, id, nwords, w[k], l[k], fixdur[k], sacdur[k], 
						# 	readsac[k,"vpeak"], readsac[k,"dX"], readsac[k,"dY"], readsac[k,"dx"], readsac[k,"dy"], 
						# 	fpos_a[k], fpos_b[k], fpos[k], FirstLast)
						resultStr <- sprintf('%i\t%i\t%i\t%i\t%f\t%f\t%i\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%i\n',  
							subjID, id, nwords, w[k], l[k], fixdur[k], sacdur[k], 
							readsac[k,"vpeak"], readsac[k,"dX"], readsac[k,"dY"], readsac[k,"dx"], readsac[k,"dy"], 
							fpos_a[k], fpos_b[k], fpos[k], FirstLast)
							
						# cat(resultStr, file=outfilename[sacdefLbls[sacdef]], append = TRUE)
						cat(resultStr, file=outfile, append = TRUE)
						flush(outfile)
					} # end of  k=nsac_start:nsac_end
				} # END OF: if res$excludeSent == 0
			} # END OF: if no saccades detected (nrow(readsac)!=0)
		} # END OF: if no missing values
	} # END OF: for trial = 1:length(tab(:,1))
	
	# mis_col_s_trial_id has 3 columns: subject nr., trial nr., sentence id
	nrMissings <- 0
	if (length(mis_col_s_trial_id) > 0) {
		idx_mis <- (mis_col_s_trial_id[,1] == s)
		nrMissings <- sum(idx_mis);
	}
	mis_col_NrPerSubject = rbind(mis_col_NrPerSubject, c(s, nrMissings))
	# cat(sprintf('%i\t%i\n',s,nrMissings), file=fout_missings_b)
	
	# collect fixations that are left of the sentence
	# while they are not the first fixation in a sentence
	nrMissings2 <- 0
	if (length(mis2_col_s_trial_id) > 0) {
		idx_mis2 <- (mis2_col_s_trial_id[,1] == s)
		nrMissings2 <- sum(idx_mis2)
	}
	mis2_col_NrPerSubject <- rbind(mis2_col_NrPerSubject, c(s, nrMissings2))
	# cat(sprintf('%i\t%i\n',s,nrMissings2), file=fout_missings2_b)
} # for (s in 1:nos)


close(outfile)
#fclose(fout);  # close file for right-eye data

# fout_missings_b <- file(paste(missingpath, exp_str, '_missings_NrPerSubject.dat', sep=''), 'w')
   # write.table(mis2_col_NrPerSubject, file=fout_missings_b)
   # close(fout_missings_b);

missings_b <- file.path(missingpath, paste(exp_str, '_missings_NrPerSubject.dat', sep=''))
write.table(mis_col_NrPerSubject, file=missings_b)

missings2_b <- file.path(missingpath, paste(exp_str, '_left_of_sent_and_not_first_NrPerSubject.dat', sep=''))
write.table(mis2_col_NrPerSubject, file=missings2_b)

missings_a <- file.path(missingpath, paste(exp_str, '_missings_s_trial_id.dat', sep=''))
write.table(mis_col_s_trial_id, file=missings_a, row.names = FALSE, col.names = FALSE)

missings2_a <- file.path(missingpath, paste(exp_str, '_left_of_sent_and_not_first_s_trial_id.dat', sep=''))
write.table(mis2_col_s_trial_id, file=missings2_a, row.names = FALSE)

missings_all <- file.path(missingpath, paste(exp_str, '_all_excluded_sent_NrPerSubject.dat', sep=''))
mis_all <- mis_col_NrPerSubject
mis_all[,2] = mis_all[,2] + mis2_col_NrPerSubject[,2]
write.table(mis_all, file=missings_all, row.names = FALSE)

cat('\n\tOk\n')

# diagnostics, for evaluating saccade detection parameters
# diagFixdur_AAA
