\name{microsacc}
\alias{microsacc}
%- Also NEED an '\alias' for EACH other topic documented here.
\encoding{UTF-8}
\title{ Microsaccade detection }
\description{
 Microsaccades and saccades are detected as outliers in velocity space by the algorithm described in Engbert & Kliegl (2003).
}
\note{
IMPORTANT: In analyses of reading data, this should always (even for monocular recordings) be followed by a call to binsacc, because (due to overshoot?) otherwise very short fixations occur between a real saccade and an overshoot component or control saccade. Binsacc combines clusters separated by fixations <=5 samples.
}
\usage{
microsacc(x, vel, VFAC=5, MINDUR=3, discardMS=TRUE, removeGlissades=TRUE, DTGLISS=30, glissadesBelongToFixation=FALSE, MSamplCrit=0.76)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{x}{ N-by-2 position vector (x,y) }
  \item{vel}{ N-by-2 velocity vector (vx, vy), e.g., output of vecvel}
  \item{VFAC}{ relative velocity threshold (multiplier of deviation threshold, defaults to 5)}
  \item{MINDUR}{ minimal saccade duration in samples, defaults to 3}
  \item{discardMS}{whether to discard microsaccades (as e.g., in reading studies), defaults to TRUE}
  \item{removeGlissades}{whether to discard glissades, defaults to TRUE}
  \item{DTGLISS}{maximum time between saccade offset and following saccade onset to count as glissade, defaults to 30 ms}
  \item{glissadesBelongToFixation}{logical value, if TRUE glissade duration is added to fixation duration (by choosing first saccade offset as ensemble offset), if FALSE ensemble saccade duration is defined from onset of first to end of last saccade, defaults to FALSE}
  \item{MSamplCrit}{minimum saccade amplitude (in degrees) if discardMS==TRUE}
}

% \details{}

\value{
 matrix describing saccade characteristics, containing one row per detected saccade
 columns:
\tabular{ll}{
          t0 \tab start time \cr
          t1 \tab end time \cr
          vpeak \tab peak velocity \cr
          dx \tab horizontal component (start to end)\cr
          dy \tab vertical component (start to end)\cr
          dX \tab horizontal amplitude (maximum deviation)\cr
          dY \tab vertical amplitude (maximum deviation)
       }
}
\references{ Engbert, R. & Kliegl, R. (2003) Microsaccades uncover the orientation of covert attention. \emph{Vision Research}, \bold{43}, 1035--1045. 

\url{http://www.agnld.uni-potsdam.de/~ralf/MS/}
}
\author{ Jochen Laubrock, Ralf Engbert, Konstantin Mergenthaler }
\note{ Current implementation of discardMS is poor, purely based on saccade amplitude. Should probably not be used or default to FALSE.}
\seealso{ \code{\link{vecvel}}, \code{\link{binsacc}} }
\examples{

data(trial4)
# compute point velocity
vl <- vecvel(trial4[,c("xl", "yl")], 500, 2)
# specify detection parameters
VFAC <- 5
MINDUR <- 3
# detect saccades in left eye 
sacl <- microsacc(trial4[,c("xl", "yl")], vl, VFAC, MINDUR, removeGlissades=FALSE)
# dto., but remove glissades
sacl2 <- microsacc(trial4[,c("xl", "yl")], vl, VFAC, MINDUR)

# visualize detection
sacl <- sacl$sac
sacl2 <- sacl2$sac

par(mfrow=c(4,1))
firstN <- 5
tmax <- sacl[firstN,2]

plot(vl[(1:tmax),1], type='l', main="velocity", xlab="time [ms]", ylab="velocity")
for (i in 1:firstN) { 
	ix <- (sacl[i,"t0"]:sacl[i,"t1"])
	points(ix, vl[ix,1], pch='o', col=i) 
}

plot(trial4[1:tmax,"xl"], type='l', main="removeGlissades=FALSE", xlab="time [ms]", ylab="x position")
for (i in 1:firstN) {
	ix <- (sacl[i,"t0"]:sacl[i,"t1"])
	points(ix, trial4[ix,"xl"], pch='o', col=i)
}

plot(trial4[1:tmax,"xl"], type='l', main="removeGlissades=TRUE", xlab="time [ms]", ylab="x position")
for (i in 1:firstN) {
	ix <- (sacl2[i,"t0"]:sacl2[i,"t1"])
	points(ix, trial4[ix,"xl"], pch='o', col=i)
}

saclb <- binsacc(sacl, sacl)
saclb <- saclb$sac[,1:7]
names(saclb) <- names(sacl)
plot(trial4[1:tmax,"xl"], type='l', main="removeGlissades=FALSE, after binsacc", xlab="time [ms]", ylab="x position")
for (i in 1:firstN) {
	ix <- (saclb[i,"t0"]:saclb[i,"t1"])
	points(ix, trial4[ix,"xl"], pch='o', col=i ) 
}


}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
% \keyword{  }
% \keyword{  }% __ONLY ONE__ keyword per line
