\name{removeGlissades}
\alias{removeGlissades}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Glissade removal 
}
\description{
Glissades are short saccades that appear in the temporal neighborhood of larger saccades. Often, glissades appear to be directed in the opposite direction of a saccade, perhaps indicating a kind of `bang-bang control' by the oculomotor system. With the availability of high-speed recording, glissades have increasingly become a nuisance for higher-level ("cognitive") analyses, as velocity falls briefly below the threshold between a saccade and a glissade and thus velocity-based saccade detection algorithms detect very short artefactual fixations. Since it is currently unclear whether to count glissade duration to a fixation or not, we provide an option to do either (see Note).
}
\usage{
removeGlissades(sac, DTGLISS = 30, glissadesBelongToFixation = TRUE)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{sac}{
	 \code{sac}: data frame of (micro-)saccades, as returned by microsacc
	must have column names c('t0', 't1', 'vpeak', 'dx', 'dy', 'dX', 'dY')
}
  \item{DTGLISS}{
 \code{DTGLISS}: maximum time between (micro-)saccade offset and following (micro-)saccade onset to count as glissade, defaults to 30 ms
}
  \item{glissadesBelongToFixation}{
	 \code{glissadesBelongToFixation}: logical value, if TRUE glissade duration is added to fixation duration (by choosing first saccade offset as ensemble offset), if FALSE ensemble saccade duration is defined from onset of first to end of last saccade, defaults to TRUE)
%%     ~~Describe \code{glissadesBelongToFixation} here~~
}
}
% \details{
% %%  ~~ If necessary, more details than the description above ~~
% }
\value{
  \item{sac}{sac data frame with glissades removed, same format as input sac: \tabular{rl}{
         t0 \tab start time \cr
         t1 \tab end time \cr
         vpeak \tab peak velocity \cr
         dx \tab x component \cr
         dy \tab y component \cr
         dX \tab x amplitude \cr
         dY \tab y amplitude
       }
}
  \item{hasGlissade }{logical vector indicating rows in sac for which glissades were removed}
}
% \references{
% %% ~put references to the literature/web site here ~
% }
\author{
	Jochen Laubrock
}
\note{
	It is unclear whether to add glissade duration to fixation or saccade duration, as there seems to be no standard procedure for glissade removal. Because of the small amplitude of glissades, we here default to the former.
}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
\code{\link{microsacc}}, \code{\link{vecvel}}
}
\examples{

data(trial4)
xyl <- trial4[,2:3]
vl <- vecvel(xyl, 500, 2)
sacl <- microsacc(xyl, vl, discardMS=FALSE, removeGlissades=FALSE)
sac <- sacl$sac
sacl2 <- removeGlissades(sac, glissadesBelongToFixation=FALSE)
sac2 <- sacl2$sac
sacl3 <- removeGlissades(sac, glissadesBelongToFixation=TRUE)
sac3 <- sacl3$sac

SHOWPLOT=TRUE
if(SHOWPLOT){
	par(mfrow=c(4,1))
	firstN <- 10
	tmax <- sac[firstN,2]

	avl <- abs(vl)
	avl[avl<0] <- min(avl[avl>0])/2
	plot(avl[(1:tmax),1], type='l', ylab='abs(velocity)', main='absolute velocity')
	for (i in 1:firstN) { 
		ix <- (sac[i,"t0"]:sac[i,"t1"])
		points(ix, avl[ix,1], pch='o', col=i) 
	}

	plot(trial4[1:tmax,"xl"], type='l', ylab='position', main='after microsacc, before removeGlissades')
	for (i in 1:firstN) {
		ix <- (sac[i,"t0"]:sac[i,"t1"])
		points(ix, trial4[ix,"xl"], pch='o', col=i)
	}

	ixs2 <- which(sac2[,"t0"]<= sac[firstN,"t0"])
	plot(trial4[1:tmax,"xl"], type='l', ylab='position', main='after removeGlissades(..., glissadesBelongToFixation=FALSE')
	for (i in seq_along(ixs2)) {
		ix <- (sac2[ixs2[i],"t0"]:sac2[ixs2[i],"t1"])
		points(ix, trial4[ix,"xl"], pch='o', col=i)
	}

	ixs3 <- which(sac3[,"t0"]<= sac[firstN,"t0"])
	plot(trial4[1:tmax,"xl"], type='l', ylab='position', main='after removeGlissades(..., glissadesBelongToFixation=TRUE) ')
	for (i in seq_along(ixs3)) {
		ix <- (sac3[ixs3[i],"t0"]:sac3[ixs3[i],"t1"])
		points(ix, trial4[ix,"xl"], pch='o', col=i)
	}
}

## The function is currently defined as
"removeGlissades"
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
% \keyword{ ~kwd1 }
% \keyword{ ~kwd2 }% __ONLY ONE__ keyword per line
